@@ -3,7 +3,16 @@
 #%stage: filesystem
 #%depends: resume
 #
-#%programs: /sbin/fsck $rootfsck $usrfsck
+#%programs: fsck
+#%programs: $rootfsck
+#%programs: $usrfsck
+#%programs: mkdir
+#%programs: mount
+#%programs: on_ac_power
+#%programs: reboot
+#%programs: showconsole
+#%programs: sed
+#%programs: udevadm
 #%if: ! "$root_already_mounted"
 #%dontshow
 #
@@ -58,12 +67,10 @@ read_only=${cmd_ro}
 
 # And now for the real thing
 if ! discover_root ; then
-    echo "not found -- exiting to /bin/sh"
-    cd /
-    PATH=$PATH PS1='$ ' /bin/sh -i
+    emergency "not found"
 fi
 
-sysdev=$(/sbin/udevadm info -q path -n $rootdev)
+sysdev=$(udevadm info -q path -n $rootdev)
 # Fallback if rootdev is not controlled by udev
 if [ $? -ne 0 ] && [ -b "$rootdev" ] ; then
     devn=$(devnumber $rootdev)
@@ -76,20 +83,19 @@ if [ $? -ne 0 ] && [ -b "$rootdev" ] ; then
     unset maj
     unset min
 fi
-if [ -z "$rootfstype" -a -x /sbin/udevadm -a -n "$sysdev" ]; then
-    eval $(/sbin/udevadm info -q env -p $sysdev | sed -n '/ID_FS_TYPE/p')
+if [ -z "$rootfstype" -a -n "$(type -p udevadm)" -a -n "$sysdev" ]; then
+    eval $(udevadm info -q env -p $sysdev | sed -n '/ID_FS_TYPE/p')
     rootfstype=$ID_FS_TYPE
     [ -n "$rootfstype" ] && [ "$rootfstype" = "unknown" ] && rootfstype=
     ID_FS_TYPE=
 fi
 
+oacp=$(type -p on_ac_power)
 # check filesystem if possible
 if [ -z "$rootfstype" ]; then
-    echo "invalid root filesystem -- exiting to /bin/sh"
-    cd /
-    PATH=$PATH PS1='$ ' /bin/sh -i
+    emergency "invalid root filesystem"
 # skip fsck if running on battery                                                                                                                                         
-elif [ -x /usr/bin/on_ac_power ] && ! /usr/bin/on_ac_power -q ; then
+elif [ -n "${oacp}" ] && ! ${oacp} -q ; then
     echo skipping fsck because running on batteries 
 # don't run fsck in the kdump kernel
 elif [ -x "$rootfsck" ] && ! [ -s /proc/vmcore ] ; then
@@ -98,7 +104,7 @@ elif [ -x "$rootfsck" ] && ! [ -s /proc/vmcore ] ; then
     # Display progress bar if possible
     fsckopts="-V -a"
     [ "$forcefsck" ] && fsckopts="$fsckopts -f"
-    console=`/sbin/showconsole`
+    console=`showconsole`
     [ "${console##*/}" = "tty1" ] && fsckopts="$fsckopts -C"
     # Check external journal for reiserfs
     [ "$rootfstype" = "reiserfs" -a -n "$journaldev" ] && fsckopts="-j $journaldev $fsckopts"
@@ -110,14 +116,29 @@ elif [ -x "$rootfsck" ] && ! [ -s /proc/vmcore ] ; then
     echo $ROOTFS_FSCK >/run/initramfs/root-fsck
     ROOTFS_FSTYPE=$rootfstype
     export ROOTFS_FSTYPE
-    if [ $ROOTFS_FSCK -gt 1 -a $ROOTFS_FSCK -lt 4 ]; then
-        # reboot needed
-        echo "fsck succeeded, but reboot is required."
-        echo "Rebooting system."
-        /bin/reboot -d -f
-    elif [ $ROOTFS_FSCK -gt 3 ] ; then
-        echo "fsck failed. Mounting root device read-only."
-        read_only=1
+    fsck_corrected=$(( $ROOTFS_FSCK & 1 ))
+    fsck_reboot=$(( $ROOTFS_FSCK & 2 ))
+    fsck_uncorrected=$(( $ROOTFS_FSCK & 4 ))
+    fsck_other=$(( $ROOTFS_FSCK & (255 << 3) ))
+    if [ $ROOTFS_FSCK -gt 1 ]; then
+        if [ $fsck_other -gt 0 ]; then
+            echo "fsck reported unhandled exit code: $ROOTFS_FSCK"
+            echo "Mounting root device read-only."
+            read_only=1
+        fi
+        if [ $fsck_corrected -gt 0 ]; then
+            echo "Filesystem errors corrected."
+        fi
+        if [ $fsck_reboot -gt 0 ]; then
+            echo "System should be rebooted."
+        fi
+        if [ $fsck_uncorrected -gt 0 ]; then
+            emergency "Filesystem errors left uncorrected."
+        fi
+        if [ $fsck_reboot -gt 0 ]; then
+            echo "Rebooting system."
+            reboot -d -f
+        fi
     else
         if [ "$read_only" ]; then
             echo "fsck succeeded. Mounting root device read-only."
@@ -140,19 +161,16 @@ echo "Mounting root $rootdev"
 # "rootflags" command line
 if [ -n "$rootflags" ] ; then
     opt="${opt},$rootflags"
-else
-    if [ -n "$rootfsopts" ] ; then
-        opt="${opt},$rootfsopts"
-    fi
+fi
+if [ -n "$rootfsopts" ] ; then
+    opt="${opt},$rootfsopts"
 fi
 
 [ -n "$rootfstype" ] && opt="${opt} -t $rootfstype"
 echo mount $opt $rootdev /root
 mount $opt $rootdev /root
 if [ $? -ne 0 ] ; then
-    echo "could not mount root filesystem -- exiting to /bin/sh"
-    cd /
-    PATH=$PATH PS1='$ ' /bin/sh -i
+    emergency "could not mount root filesystem"
 fi
 
 unset discover_root
