@@ -4,7 +4,7 @@
 #%provides: dmroot
 #
 
-if [ -x /sbin/dmraid -a -x /sbin/dmsetup ] ; then
+if [ -x "$(type -p dmsetup)" ] ; then
 	newbd=
 	for bd in $blockdev ; do
 	    update_blockdev $bd
