@@ -423,7 +423,7 @@ class GNUTranslations(NullTranslations):
 def find(domain, localedir=None, languages=None, all=0):
     if localedir in [None, _default_localedir]:
         bundle = find(domain, localedir=_default_bundlelocaledir, languages=languages, all=all)
-        if len(bundle):
+        if bundle:
             return bundle
     # Get some reasonable defaults for arguments that were not supplied
     if localedir is None:
