@@ -25,6 +25,7 @@ textdomain("dns-server");
 YaST::YCP::Import ("Hostname");
 YaST::YCP::Import ("String");
 YaST::YCP::Import ("Mode");
+YaST::YCP::Import ("Ldap");
 
 #use io_routines;
 #use check_routines;
@@ -690,16 +691,22 @@ sub ZoneFileWriteLdap {
     }
     y2debug ("New base record: ".Dumper(\%ldap_record));
 
+    my $ldap_write = 1;
     if (scalar (@{$found_ref || []}) == 0)
     {
 	y2milestone ("Creating new zone record");
-	SCR->Write (".ldap.add", \%ldap_cmd, \%ldap_record);
+	$ldap_write = SCR->Write(".ldap.add", \%ldap_cmd, \%ldap_record);
     }
     else
     {
 	y2milestone ("Modifying existing zone record");
 	delete $ldap_record{"objectClass"}; # objectclass can be changed by mail-server
-	SCR->Write (".ldap.modify", \%ldap_cmd, \%ldap_record);
+	$ldap_write = SCR->Write(".ldap.modify", \%ldap_cmd, \%ldap_record);
+    }
+    unless ($ldap_write) {
+        y2error("LDAP Error (write): ".Ldap->LDAPError());
+        Ldap->LDAPErrorMessage('write', Ldap->LDAPError());
+        return 0;
     }
 
     my @all_records = map {
@@ -741,7 +748,13 @@ sub ZoneFileWriteLdap {
     foreach my $d (@deleted)
     {
 	y2milestone ("Removing all records regarding $d");
-	SCR->Write (".ldap.delete", {"dn" => "relativeDomainName=$d,$zone_dn"});
+	$ldap_write = SCR->Write(".ldap.delete", {"dn" => "relativeDomainName=$d,$zone_dn"});
+	last unless $ldap_write; # skip the rest
+    }
+    unless ($ldap_write) {
+        y2error("LDAP Error (delete): ".Ldap->LDAPError());
+        Ldap->LDAPErrorMessage('write', Ldap->LDAPError());
+        return 0;
     }
 
     # write all the other records
@@ -797,14 +810,22 @@ sub ZoneFileWriteLdap {
 
 	if (scalar (@{$found_ref || []}) == 0)
 	{
-	    SCR->Write (".ldap.add", \%ldap_cmd, \%ldap_record);
+	    $ldap_write = SCR->Write (".ldap.add", \%ldap_cmd, \%ldap_record);
+	    last unless $ldap_write; # skip the rest
 	}
 	else
 	{
-	    SCR->Write (".ldap.modify", \%ldap_cmd, \%ldap_record);
+	    $ldap_write = SCR->Write (".ldap.modify", \%ldap_cmd, \%ldap_record);
+	    last unless $ldap_write; # skip the rest
 	}
     }
-    return 1;
+    unless ($ldap_write) {
+        y2error("LDAP Error (write): ".Ldap->LDAPError());
+        Ldap->LDAPErrorMessage('write', Ldap->LDAPError());
+        return 0;
+    }
+
+    return $ldap_write;
 }
 
 BEGIN { $TYPEINFO{ZonesDeleteLdap} = ["function", "boolean", [ "list", "string",]];}
@@ -821,14 +842,24 @@ sub ZonesDeleteLdap {
 	@check_zones == 0;
     } @current_zones_in_ldap;
 
+    my $ldap_write = 1;
+
     foreach my $z (@zones_to_delete) {
 	y2milestone ("Removing zone $z from LDAP");
 	my %request = (
 	    "dn" => "zoneName=$z,$zone_base_config_dn",
 	    "subtree" => 1,
 	);
-	SCR->Write (".ldap.delete", \%request);
+	$ldap_write = SCR->Write (".ldap.delete", \%request);
+
+	unless ($ldap_write) {
+	    y2error("LDAP Error (write): ".Ldap->LDAPError());
+	    Ldap->LDAPErrorMessage('write', Ldap->LDAPError());
+	    last;
+	}
     }
+
+    return $ldap_write;
 }
 
 BEGIN{ $TYPEINFO{ZonesListLdap} = ["function", ["list", "string"]];}
