@@ -1,10 +1,30 @@
 #! /usr/bin/perl -w
+# ------------------------------------------------------------------------------
+# Copyright (c) 2006-2012 Novell, Inc. All Rights Reserved.
+#
+#
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of version 2 of the GNU General Public License as published by the
+# Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, contact Novell, Inc.
+#
+# To contact Novell about this file by physical or electronic mail, you may find
+# current contact information at www.novell.com.
+# ------------------------------------------------------------------------------
+#
+
 #
 # File:		modules/UsersSimple.pm
 # Package:	Configuration of users and groups
 # Summary:	module for first stage user configuration
 #
-# $Id: UsersSimple.pm 61042 2010-02-26 13:48:03Z jsuchome $
+# $Id$
 #
 
 package UsersSimple;
@@ -47,7 +67,7 @@ my @users			= ();
 
 
 # password encryption method
-my $encryption_method		= "blowfish";
+my $encryption_method		= "sha512";
 
 # mail alias for root
 my $root_alias			= "";
@@ -70,6 +90,8 @@ my %max_lengths			= (
     "des"	=> 8,
     "md5"	=> 127,
     "blowfish"	=> 72,
+    "sha256"	=> 127, # arbitrary high number, there's probably no limit
+    "sha512"	=> 127
 );
 
 # name of user that should be logged in automatically
@@ -90,7 +112,7 @@ my $character_class 		= "[[:alpha:]_][[:alnum:]_.-]*[[:alnum:]_.\$-]\\?";
 my $max_length_login 	= 32; # reason: see for example man utmp, UT_NAMESIZE
 my $min_length_login 	= 2;
 
-# see SYSTEM_UID_MAX and SYSTEM_GID_MAX in /etc/login.defs
+# see SYS_UID_MAX and SYS_GID_MAX in /etc/login.defs
 my $max_system_uid	= 499;
 
 # maps for user data read in 1st stage ('from previous installation')
@@ -218,6 +240,14 @@ sub bool {
     return $param;
 }
 
+##------------------------------------
+# set new value for $character_class
+BEGIN { $TYPEINFO{SetCharacterClass} = ["function", "void", "string"];}
+sub SetCharacterClass {
+    my $self            = shift;
+    $character_class    = shift;
+}
+
 ##------------------------------------
 # set new cracklib dictionary path
 BEGIN { $TYPEINFO{SetCrackLibDictPath} = ["function", "void", "string"];}
@@ -641,7 +671,7 @@ sub CheckObscurity {
     $filtered 		= reverse $pw;
     if ($filtered eq $pw) {
 	# popup question
-        return __("You have used a palindrom for the password.");
+        return __("You have used a palindrome for the password.");
     }
 
     # check for numbers
@@ -949,7 +979,9 @@ sub Read {
     my $file	= Directory->vardir()."/users_first_stage.ycp";
     my $ret	= 0;
 
-    if (FileUtils->Exists ($file) && ($force || $first_stage_data_not_read)) {
+    if (Stage->cont () && FileUtils->Exists ($file) &&
+	($force || $first_stage_data_not_read)) 
+    {
 	my $data	= SCR->Read (".target.ycp", $file);
 	$first_stage_data_not_read	= 0;
 	if (defined $data && ref ($data) eq "HASH") {
